import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:hub/actions/actions.dart';
import 'package:hub/app_state.dart';
import 'package:hub/backend/api_requests/api_calls.dart';
import 'package:hub/backend/api_requests/api_manager.dart';
import 'package:hub/components/templates_components/card_item_template_component/card_item_template_component_widget.dart';
import 'package:hub/components/templates_components/details_component/details_component_action.dart';
import 'package:hub/flutter_flow/flutter_flow_theme.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/internationalization.dart';
import 'package:hub/pages/schedule_complete_visit_page/schedule_complete_visit_page_model.dart';
import 'package:provider/provider.dart';

class VisitHistoryWidget extends StatefulWidget {
  VisitHistoryWidget({
    Key? key,
  }) : super(key: key);

  @override
  _VisitHistoryWidgetState createState() => _VisitHistoryWidgetState();
}

class _VisitHistoryWidgetState extends State<VisitHistoryWidget> {
  @override
  Widget build(BuildContext context) {
    var response = ScheduleCompleteVisitPageModel().visitHistory(
      requestFn: () => PhpGroup.getVisitsCall.call(
        devUUID: FFAppState().devUUID,
        userUUID: FFAppState().userUUID,
        cliID: FFAppState().cliUUID,
        atividade: 'getVisitas',
        pageSize: 100,
        pageNumber: 1,
      ),
    );
    return Container(
      width: double.infinity,
      height: 900.0,
      decoration: BoxDecoration(
        color: FlutterFlowTheme.of(context).primaryBackground,
      ),
      // child: Consumer<ScheduleCompleteVisitPageModel>(
      child: StreamBuilder<dynamic>(
        stream: response,
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.waiting) {
            return Center(
              child: CircularProgressIndicator(
                valueColor: AlwaysStoppedAnimation<Color>(
                  FlutterFlowTheme.of(context).primary,
                ),
              ),
            );
          } else if (snapshot.hasError) {
            return Center(child: Text('Error: ${snapshot.error}'));
          } else if (!snapshot.hasData || snapshot.data!.jsonBody == null) {
            return const Center(child: Text('No visits found'));
          }
          var wrapGetVisitsResponse = snapshot.data!;
          var visitaWrap = PhpGroup.getVisitsCall
                  .visitasList(wrapGetVisitsResponse.jsonBody)
                  ?.toList() ??
              [];
          log('updated');
          log('visitaWrap: ${visitaWrap[0]}');
          return ListView.builder(
            itemCount: visitaWrap.length,
            shrinkWrap: true,
            physics: const BouncingScrollPhysics(),
            itemBuilder: (context, index) {
              final visitaWrapItem = visitaWrap[index];
              return CardItemTemplateComponentWidget(
                imageHashMap: {
                  'key': visitaWrapItem['VTE_DOCUMENTO'] ?? '',
                  'value': 'E',
                },
                labelsHashMap: {
                  'Nome:': visitaWrapItem['VTE_NOME'] ?? '',
                  'Inicio:': visitaWrapItem['VAW_DTINICIO'] ?? '',
                  'Fim:': visitaWrapItem['VAW_DTFIM'] ?? '',
                },
                statusHashMap: [
                  if (getStatus(visitaWrapItem['VAW_STATUS']) == status.active)
                    {
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Ativo',
                        enText: 'Active',
                      ): FlutterFlowTheme.of(context).warning,
                    },
                  if (getStatus(visitaWrapItem['VAW_STATUS']) ==
                      status.finished)
                    {
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Finalizado',
                        enText: 'Finished',
                      ): FlutterFlowTheme.of(context).success,
                    },
                  if (getStatus(visitaWrapItem['VAW_STATUS']) == status.unknown)
                    {
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Desconhecido',
                        enText: 'Unknown',
                      ): FlutterFlowTheme.of(context).alternate,
                    },
                  if (getStatus(visitaWrapItem['VAW_STATUS']) ==
                      status.canceled)
                    {
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Cancelado',
                        enText: 'Canceled',
                      ): FlutterFlowTheme.of(context).error,
                    },
                  if (getStatus(visitaWrapItem['VAW_STATUS']) == status.blocked)
                    {
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Bloqueado',
                        enText: 'Blocked',
                      ): FlutterFlowTheme.of(context).error,
                    },
                  if (getStatus(visitaWrapItem['VAW_STATUS']) ==
                      status.inactive)
                    {
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Inactive',
                        enText: 'Inactive',
                      ): FlutterFlowTheme.of(context).error,
                    },
                ],
                onTapCardItemAction: () async {
                  await showDialog(
                    // isScrollControlled: true,
                    // isDismissible: true,
                    // backgroundColor: Colors.transparent,
                    useSafeArea: true,
                    context: context,

                    builder: (context) {
                      return Dialog(
                        alignment: Alignment.center,
                        child: buildDetails(
                          visitaWrapItem,
                          context,
                          changeStatusAction,
                        ),
                      );
                    },
                  ).whenComplete(() async {
                    log('Complete');
                    final newResponseStream =
                        ScheduleCompleteVisitPageModel().visitHistory(
                      requestFn: () => PhpGroup.getVisitsCall.call(
                        devUUID: FFAppState().devUUID,
                        userUUID: FFAppState().userUUID,
                        cliID: FFAppState().cliUUID,
                        atividade: 'getVisitas',
                        pageSize: 100,
                        pageNumber: 1,
                      ),
                    );
                    newResponseStream.listen((newResponse) {
                      log('New response received: $newResponse');
                      if (mounted) {
                        safeSetState(() {
                          response = newResponse.jsonBody;
                          log('Response updated: $response');
                        });
                      } else {
                        log('Widget is not mounted, skipping state update.');
                      }
                    }).onError((err) {
                      log('Error: $err');
                    });
                  }).catchError((err, stack) {
                    log('Error: $err');
                    log('Stack: $stack');
                  });
                },
              );
            },
          );
        },
      ),
    );
  }
}
